var StreamFeed = require('../../../src/lib/feed'),
    expect = require('expect.js'),
    beforeEachFn = require('../utils/hooks').beforeEach,
    td = require('testdouble'),
    stream = require('../../../src/getstream'),
    signing = require('../../../src/lib/signing');


describe('[UNIT] Stream Client (Node)', function() {

    beforeEach(beforeEachFn);

    it('#updateActivities', function() {
        var self = this;

        expect(function() {
            self.client.updateActivities('A-String-Thing');
        }).to.throwException(function(e) {
            expect(e).to.be.a(TypeError);
        });
    });

    it('#userAgent', function() {
        var useragent = this.client.userAgent();

        expect(useragent).to.be('stream-javascript-client-node-unknown');
    });

    it('#feed', function() {
        var feed = this.client.feed('user', 'jaap', '123456789');

        expect(feed).to.be.a(StreamFeed);
    });

    describe('#enrichUrl', function() {

        it('(1) personalization service', function() {
            var resource = 'influencers'
            var url = this.client.enrichUrl(resource, 'personalization');
            expect(url).to.be('https://personalization.stream-io-api.com/personalization/' + this.client.version + '/' + resource);
        });
    });

    describe('#updateActivities', function() {

        it('throws', function() {
            function isGoingToThrow1() {
                this.client.updateActivities({});
            }

            function isGoingToThrow2() {
                this.client.updateActivities(0);
            }

            function isGoingToThrow3() {
                this.client.updateActivities(null);
            }

            function isNotGoingToThrow() {
                this.client.updateActivities([]);
            }

            function isTypeError(err) {
                expect(err).to.be.a(TypeError);
            }

            expect(isGoingToThrow1).to.throwException(isTypeError);
            expect(isGoingToThrow2).to.throwException(isTypeError);
            expect(isGoingToThrow3).to.throwException(isTypeError);
            expect(isNotGoingToThrow).to.not.throw;
        });

        it('(1) works', function() {
            var post = td.function();
            td.replace(this.client, 'post', post);

            var activities = [{ actor: 'matthisk', object: 0, verb: 'do' }];

            this.client.updateActivities(activities);

            td.verify(post(td.matchers.contains({
                url: 'activities/',
            }), undefined));
        });

        it('(2) works - callback', function() {
            var post = td.function();
            td.replace(this.client, 'post', post);

            var activities = [{ actor: 'matthisk', object: 0, verb: 'do' }];
            var fn = function() {};

            this.client.updateActivities(activities, fn);

            td.verify(post(td.matchers.contains({
                url: 'activities/',
            }), fn));
        });

        it('(3) update single activity', function() {
            var post = td.function();
            td.replace(this.client, 'post', post);

            var activities = [{ actor: 'matthisk', object: 0, verb: 'do' }];

            this.client.updateActivity(activities[0]);

            td.verify(post(td.matchers.contains({
                url: 'activities/',
            }), undefined));
        });

        it('(3) update single activity - callback', function() {
            var fn = function() {};
            var post = td.function();
            td.replace(this.client, 'post', post);

            var activities = [{ actor: 'matthisk', object: 0, verb: 'do' }];

            this.client.updateActivity(activities[0], fn);

            td.verify(post(td.matchers.contains({
                url: 'activities/',
            }), fn));
        });

    });

    describe('#getActivities', function() {

        it('throws', function() {
            function isGoingToThrow1() {
                this.client.getActivities({});
            }

            function isGoingToThrow2() {
                this.client.getActivities(0);
            }

            function isGoingToThrow3() {
                this.client.getActivities(null);
            }

            function isNotGoingToThrow() {
                this.client.getActivities([]);
            }

            function isTypeError(err) {
                expect(err).to.be.a(TypeError);
            }

            expect(isGoingToThrow1).to.throwException(isTypeError);
            expect(isGoingToThrow2).to.throwException(isTypeError);
            expect(isGoingToThrow3).to.throwException(isTypeError);
            expect(isNotGoingToThrow).to.not.throw;
        });

        describe('by ID', function() {

            it('(1) works', function() {
                var get = td.function();
                td.replace(this.client, 'get', get);

                var ids = ['one', 'two', 'three'];

                this.client.getActivities({ids: ids});

                td.verify(get(td.matchers.contains({
                    url: 'activities/',
                }), undefined));
            });

            it('(2) works - callback', function() {
                var get = td.function();
                td.replace(this.client, 'get', get);

                var ids = ['one', 'two', 'three'];
                var fn = function() {};

                this.client.getActivities({ids: ids}, fn);

                td.verify(get(td.matchers.contains({
                    url: 'activities/',
                }), fn));
            });
        });

        describe('by foreign ID and time', function () {

            it('(1) works', function () {
                var get = td.function();
                td.replace(this.client, 'get', get);

                var foreignIDTimes = [
                    { foreignID: "like:1", time: "2018-07-08T14:09:36.000000"},
                    { foreignID: "post:2", time: "2018-07-09T20:30:40.000000"}
                ];

                this.client.getActivities({ foreignIDTimes: foreignIDTimes });

                td.verify(get(td.matchers.contains({
                    url: 'activities/',
                }), undefined));
            });

            it('(2) works - callback', function () {
                var get = td.function();
                td.replace(this.client, 'get', get);

                var foreignIDTimes = [
                    { foreignID: "like:1", time: "2018-07-08T14:09:36.000000" },
                    { foreignID: "post:2", time: "2018-07-09T20:30:40.000000" }
                ];
                var fn = function () { };

                this.client.getActivities({ foreignIDTimes: foreignIDTimes }, fn);

                td.verify(get(td.matchers.contains({
                    url: 'activities/',
                }), fn));
            });
        });

    });
    
    describe('connect', function() {

        it('#LOCAL', function() {
            process.env['LOCAL'] = 1;

            var client = stream.connect('12345', 'abcdefghijklmnop');
            expect(client.baseUrl).to.be('http://localhost:8000/api/');

            delete process.env['LOCAL'];
        });

        it('#LOCAL', function() {
            var client = stream.connect('12345', 'abcdefghijklmnop', null, {
                location: 'nl-NL'
            });
            expect(client.baseUrl).to.be('https://nl-NL-api.stream-io-api.com/api/');
        });

        it('#LOCAL_FAYE', function() {
            process.env['LOCAL_FAYE'] = 1;

            var client = stream.connect('12345', 'abcdefghijklmnop');
            expect(client.fayeUrl).to.be('http://localhost:9999/faye/');

            delete process.env['LOCAL_FAYE'];
        });

        it('#STREAM_BASE_URL', function() {
            process.env['STREAM_BASE_URL'] = 'https://local.stream-io-api.com/api/';

            var client = stream.connect('12345', 'abcdefghijklmnop');
            expect(client.baseUrl).to.be('https://local.stream-io-api.com/api/');

            delete process.env['STREAM_BASE_URL'];
        });

    });
});
